package net.alviano.safe;

import static org.apache.commons.lang3.Validate.*;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public final class Concessionaria {
	private Id nextId = new Id(1); 
	private final List<Veicolo> veicoli = new ArrayList<Veicolo>();
	
	public void aggiungiAutomobile(Targa targa, CasaProduttrice casaProduttrice, ModelloVeicolo modello, Prezzo prezzo) {
		this.veicoli.add(new Automobile(nextId, targa, casaProduttrice, modello, prezzo));
		nextId = nextId.next();
	}
	
	public void aggiungiMoto(Targa targa, CasaProduttrice casaProduttrice, ModelloVeicolo modello, Prezzo prezzo) {
		this.veicoli.add(new Moto(nextId, targa, casaProduttrice, modello, prezzo));
		nextId = nextId.next();
	}
	
	public void rimuoviVeicolo(Veicolo v) {
		this.veicoli.remove(v);
	}
	
	public List<Veicolo> veicoli() {
		return Collections.unmodifiableList(veicoli);
	}
	
	public void ordinaVeicoliPerPrezzo() {
		this.veicoli.sort(new Comparator<>() {
			@Override
			public int compare(Veicolo arg0, Veicolo arg1) {
				return arg0.prezzo().compareTo(arg1.prezzo());
			}
		});
	}
	
	public void ordinaVeicoliPerNome() {
		this.veicoli.sort(new Comparator<>() {
			@Override
			public int compare(Veicolo arg0, Veicolo arg1) {
				return arg0.modello().compareTo(arg1.modello());
			}
		});
	}
	
	public Prezzo calcolaValore() {
		Prezzo res = new Prezzo(0);
		for(Veicolo v : this.veicoli) {
			res = res.add(v.calcolaPrezzoFinale());
		}
		return res;
	}
	
	public void salvaSuFile(String filename) throws IOException {
		List<String> list = new ArrayList<String>();
		for(Veicolo v : this.veicoli) {
			list.add(v.toSaveFormat());
		}
		Files.write(Paths.get(filename), list);
	}
	
	public void leggiDaFile(String filename) throws IOException {
		this.veicoli.clear();
		List<String> lines = Files.readAllLines(Paths.get(filename));
		for(String line : lines) {
			String[] atts = line.split(";");
			Id id = Id.parse(atts[0]);
			String tipo = atts[1];
			Targa targa = new Targa(atts[2]);
			CasaProduttrice casa = new CasaProduttrice(atts[3]);
			ModelloVeicolo nome = new ModelloVeicolo(atts[4]);
			Prezzo prezzo = Prezzo.parse(atts[5]);
			switch (tipo) {
			case "Automobile": {
				this.veicoli.add(new Automobile(id, targa, casa, nome, prezzo));
				break;
			}
			case "Moto": {
				this.veicoli.add(new Moto(id, targa, casa, nome, prezzo));
				break;
			}
			default:
				throw new IllegalStateException("Concessionaria: Valore invalido durante il caricamento dei dati");
			}
			this.nextId = Id.max(this.nextId, id.next());
		}
	}
	
	public Veicolo selezionaVeicolo(Id id) {
		notNull(id);
		for(Veicolo v : veicoli) {
			if(v.id().equals(id)) return v;
		}
		throw new IllegalArgumentException("Concessionaria: id non trovato nei veicoli");
	}
	
}
